<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use FiloBlu\Refilo\Remote\Entity\MagentoToRefiloProductAdapter;
use FiloBlu\Refilo\Remote\Entity\Product;
use LogicException;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\Api\SearchCriteriaBuilderFactory;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;

/**
 * Provider implements also a SPL iterator interface
 *  in foreach() the calling order<br>
 *  -> rewind<br>
 *  -> valid<br>
 *  -> current<br>
 *  -> key<br>
 * Class ProductProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
class ProductProvider implements EntityProviderInterface
{
    /** @var int */
    protected $pageSize = 1000;
    /** @var array */
    protected $items = [];
    /** @var SearchCriteriaInterface */
    protected $searchCriteria;
    /** @var StoreInterface */
    protected $store;
    /** @var array */
    protected $ids;
    /** @var false|float */
    protected $totalPages;
    /** @var MagentoToRefiloProductAdapter */
    private $productAdapter;
    /** @var ProductRepository */
    private $productRepository;
    /** @var Status */
    private $status;
    /** @var SearchCriteriaBuilderFactory */
    private $searchCriteriaBuilderFactory;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * ProductProvider constructor.
     * @param MagentoToRefiloProductAdapter $productAdapter
     * @param SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory
     * @param ProductRepository $productRepository
     * @param Status $status
     */
    public function __construct(
        MagentoToRefiloProductAdapter $productAdapter,
        SearchCriteriaBuilderFactory $searchCriteriaBuilderFactory,
        ProductRepository $productRepository,
        Status $status,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        $this->productAdapter = $productAdapter;
        $this->productRepository = $productRepository;
        $this->status = $status;
        $this->searchCriteriaBuilderFactory = $searchCriteriaBuilderFactory;
        $this->scopeConfig = $scopeConfig;

    }

    /**
     * @param array $ids
     * @return $this|EntityProviderInterface
     */
    public function withIds(array $ids): EntityProviderInterface
    {
        $this->ids = $ids;
        return $this;
    }

    /**
     * @param StoreInterface $store
     * @return $this|EntityProviderInterface
     */
    public function forStore(StoreInterface $store): EntityProviderInterface
    {
        $this->store = $store;
        return $this;
    }

    /**
     * @param WebsiteInterface $website
     * @return EntityProviderInterface
     */
    public function forWebsite(WebsiteInterface $website): EntityProviderInterface
    {
        throw new LogicException('Count not filter by website');
    }

    /**
     * @return mixed|void
     */
    public function current()
    {
        return $this->productAdapter->adapt(current($this->items), new Product());
    }

    /**
     * @return mixed|void
     */
    public function next()
    {
        return next($this->items);
    }

    /**
     * @return bool|float|int|string|null
     */
    public function key()
    {
        return current($this->items)->getId();
    }

    /**
     * @return bool
     */
    public function valid()
    {

        if ($this->searchCriteria->getCurrentPage() < $this->totalPages) {
            if (current($this->items) !== false) {
                return true;
            }
            $this->searchCriteria->setCurrentPage($this->searchCriteria->getCurrentPage() + 1);
            $result = $this->productRepository->getList($this->searchCriteria);
            $this->items = $result->getItems();
            return true;
        }

        return current($this->items) !== false;
    }

    /**
     *
     */
    public function rewind()
    {

        $page_size = (int) $this->scopeConfig->getValue("sync/data/batch_size");
        $this->pageSize = ($page_size) ? $page_size : $this->pageSize;

        $builder = $this->searchCriteriaBuilderFactory->create();

        // $builder->addFilter('status', $this->status->getVisibleStatusIds(), 'in');

        if (!empty($this->ids)) {
            $builder->addFilter('entity_id', $this->ids, 'in');
        }

        if ($this->store !== null) {
            $builder->addFilter('store_id', $this->store->getId(), 'eq');
        }

        $this->searchCriteria = $builder->create();
        $this->searchCriteria->setPageSize($this->pageSize)->setCurrentPage(1);
        $result = $this->productRepository->getList($this->searchCriteria);

        $this->totalPages = ceil($result->getTotalCount() / $this->pageSize);

        $this->items = $result->getItems();
    }
}
